<?php

namespace OPDashboard\Customers;

use NumberFormatter;

use OPDashboard\Services\DataObject;

class PricingPlan extends DataObject
{
    /**
     * Return formatted pricing plan amount.
     *
     * Uses PECL NumberFormatter if "ext-intl" is installed, Symfony's Intl
     * NumberFormatter if OP Checkouts is enabled and locale is set to "en" (as
     * it only supports "en" at the moment), or it falls back to
     * "number_format_i18n".
     *
     * @return  string
     */
    public function getFormattedAmountAttribute()
    {
        $locale = get_locale();

        if (extension_loaded('intl')) {
            $fmt = new NumberFormatter($locale, NumberFormatter::CURRENCY);

            return $fmt->formatCurrency($this->amount / 100, $this->currency);
        }

        return '$' . number_format_i18n($this->amount / 100, 2);
    }
}
