<?php

namespace OPDashboard\Providers;

use OPDashboard\Http\OAuthController;
use OPDashboard\Services\Route;

class RouteRegistration
{
    /**
     * Register plugin routes
     *
     * @return void
     */
    public function register()
    {
        add_action('template_redirect', [new OAuthController, 'routes']);
        add_action('wp_loaded',         [$this, 'addAdminRoutes']);
        add_action('wp_loaded',         [$this, "addWebRoutes"]);
        add_action('admin_menu',        [$this, "registerRoutes"]);
        add_action('rest_api_init',     [$this, 'registerApiRoutes']);

        add_filter('admin_title', [$this, 'fixMissingTitle'], 10, 2);

        $this->registerAjaxRoutes();
    }

    /**
     * Include admin route definitions
     *
     * @return void
     */
    public function addAdminRoutes()
    {
        require OPD_PATH.'/routes/admin.php';
    }

    /**
     * Include web route definitions
     *
     * @return void
     */
    public function addWebRoutes()
    {
        require OPD_PATH.'/routes/web.php';
    }

    /**
     * Register all added routes and admin pages
     *
     * @return void
     */
    public function registerRoutes()
    {
        Route::registerAdminPages();
    }

    /**
     * Include API route definitions
     *
     * @return void
     */
    public function registerApiRoutes()
    {
        require OPD_PATH.'/routes/api.php';
    }

    /**
     * All ajax routes
     *
     * @return void
     */
    public function registerAjaxRoutes()
    {
        require OPD_PATH.'/routes/ajax.php';
    }

    /**
     * Replace missing "inner" OP page title with one from route registration.
     *
     * Missing title occures due to subpage not having a parent (in order for
     * it not to be visible in the main admin WP menu).
     *
     * It attaches to "admin_title" filter from "wp-admin/admin-header.php".
     *
     * Uses global $hook_suffix to locate current page and to get title
     * reference from OP custom routes.
     *
     * @param   string  $adminTitle
     * @param   string  $title
     *
     * @return  string
     */
    public function fixMissingTitle($adminTitle, $title)
    {
        // If title is set leave early
        if ( ! empty($title)) {
            return $adminTitle;
        }

        global $hook_suffix;

        // If hook suffix is not what were after leave early
        if (strpos($hook_suffix, 'admin_page_op') !== 0) {
            return $adminTitle;
        }

        return Route::getMissingSubPageTitle(str_replace('admin_page_', '', $hook_suffix)) . $adminTitle;
    }
}
