<?php

namespace OPDashboard\Templates;

use OptimizePress\Support\Collection;
use OPDashboard\Services\DataObject;
use OPDashboard\SL\SL;

class TemplateCategory extends DataObject
{
    /**
     * Collection of all category templates
     *
     * @var Collection
     */
    protected $templates;

    /**
     * Return all category templates
     *
     * @return Collection
     */
    public function templates()
    {
        // Load templates from SL if empty
        if (! $this->templates or ! count($this->templates)) {
            $this->templates = $this->loadTemplates();
        }

        return $this->templates ?: new TemplatesCollection;
    }

    /**
     * Load templates from SL
     *
     * @return Collection
     */
    public function loadTemplates()
    {
        $this->templates = SL::getTemplates($this->uid);

        return $this->templates;
    }

    /**
     * Convert the collection to an array
     *
     * @return array
     */
    public function toArray()
    {
        $data = parent::toArray();

        // Add templates
        if (isset($this->templates) and $this->templates) {
            $data['templates'] = $this->templates->toArray();
        }

        return $data;
    }

    /**
     * Return upgrade button text
     *
     * @return string
     */
    public function upgradeButtonText(): string
    {
        return (string) $this->upgrade_button_text ?: __('Upgrade to Suite', 'optimizepress3');
    }

    /**
     * Return upgrade message
     *
     * @return string
     */
    public function upgradeMessage(): string
    {
        return (string) $this->upgrade_message ?: __('Upgrade to higher tier to use this category', 'optimizepress3');
    }

    /**
     * Return URL to upgrade screen
     *
     * @return string
     */
    public function upgradeUrl()
    {
        return 'https://www.optimizepress.com/suite-templates';
    }

    /**
     * Check if category is restricted
     *
     * @return bool
     */
    public function isRestricted()
    {
        // Get customer
        $customer = SL::getCustomerDetails();

        // Get user tier
        $customerTier = $customer->tier->uid;

        // Get category tiers
        $categoryTiers = (array) $this->tiers;

        // And compare with category tiers
        // We check if the customer tier is in category tiers, but also if there's no category tiers
        // we let the user see the templates
        if (($customerTier && in_array($customerTier, $categoryTiers)) || empty($categoryTiers)) {
            return false;
        }

        return true;
    }
}
