<?php

namespace OptimizePress\Integrations\Services\Membership;

use OptimizePress\Integrations\Collections\IntegrationFieldsCollection;
use OptimizePress\Integrations\LeadOptions;
use OptimizePress\Integrations\OptinData;
use OptimizePress\Integrations\OptinResult;
use OptimizePress\Integrations\Services\Exceptions\OptinException;
use OptimizePress\Support\Collection;

interface MembershipIntegrationInterface
{
    /**
     * Get all levels for the membership integration
     *
     * @return Collection
     */
    public function getLevels(): Collection;

    /**
     * Get all form fields for the membership integration
     *
     * @return IntegrationFieldsCollection
     */
    public function getFields(): IntegrationFieldsCollection;

    /**
     * User gets access to specific membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return OptinResult
     * @throws OptinException
     */
    public function grantLevelAccess(OptinData $optinData, LeadOptions $leadOptions = null) :OptinResult;

    /**
     * Revoke access for user to membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return bool
     */
    public function revokeLevelAccess(OptinData $optinData, LeadOptions $leadOptions = null) :bool;

    /**
     * User gets access to specific membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return OptinResult
     * @throws OptinException
     */
    public function optin(OptinData $optinData, LeadOptions $leadOptions = null) :OptinResult;
}
